@@ -23,6 +23,7 @@ GtkWidget *do_entry_completion (GtkWidget *do_widget);
 GtkWidget *do_expander (GtkWidget *do_widget);
 GtkWidget *do_hypertext (GtkWidget *do_widget);
 GtkWidget *do_iconview (GtkWidget *do_widget);
+GtkWidget *do_iconview_edit (GtkWidget *do_widget);
 GtkWidget *do_images (GtkWidget *do_widget);
 GtkWidget *do_list_store (GtkWidget *do_widget);
 GtkWidget *do_menus (GtkWidget *do_widget);
@@ -49,6 +50,12 @@ Demo child1[] = {
   { NULL } 
 };
 
+Demo child2[] = {
+  { "Icon View Basics", "iconview.c", do_iconview, NULL },
+  { "Editing and Drag-and-Drop", "iconview_edit.c", do_iconview_edit, NULL },
+  { NULL } 
+};
+
 Demo testgtk_demos[] = {
   { "Application main window", "appwindow.c", do_appwindow, NULL }, 
   { "Button Boxes", "button_box.c", do_button_box, NULL }, 
@@ -60,7 +67,7 @@ Demo testgtk_demos[] = {
   { "Drawing Area", "drawingarea.c", do_drawingarea, NULL }, 
   { "Entry Completion", "entry_completion.c", do_entry_completion, NULL }, 
   { "Expander", "expander.c", do_expander, NULL }, 
-  { "Icon View", "iconview.c", do_iconview, NULL }, 
+  { "Icon View", NULL, NULL, child2 }, 
   { "Images", "images.c", do_images, NULL }, 
   { "Menus", "menus.c", do_menus, NULL }, 
   { "Paned Widgets", "panes.c", do_panes, NULL }, 
