@@ -27,8 +27,10 @@
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
+#define GDK_PIXBUF_COMPILATION
 #include "gdk-pixbuf.h"
 #include "gdk-pixbuf-private.h"
+#include "gdk-pixbuf-alias.h"
 
 static void gdk_pixbuf_class_init   (GdkPixbufClass *klass);
 static void gdk_pixbuf_finalize     (GObject        *object);
@@ -93,17 +95,24 @@ gdk_pixbuf_class_init (GdkPixbufClass *klass)
         object_class->finalize = gdk_pixbuf_finalize;
         object_class->set_property = gdk_pixbuf_set_property;
         object_class->get_property = gdk_pixbuf_get_property;
-        
+
+#define PIXBUF_PARAM_FLAGS G_PARAM_READWRITE|G_PARAM_CONSTRUCT_ONLY|\
+                           G_PARAM_STATIC_NAME|G_PARAM_STATIC_NICK|G_PARAM_STATIC_BLURB
+        /**
+         * GdkPixbuf:n-channels:
+         *
+         * The number of samples per pixel. 
+         * Currently, only 3 or 4 samples per pixel are supported.
+         */
         g_object_class_install_property (object_class,
                                          PROP_N_CHANNELS,
-                                         g_param_spec_int ("n_channels",
+                                         g_param_spec_int ("n-channels",
                                                            P_("Number of Channels"),
                                                            P_("The number of samples per pixel"),
                                                            0,
                                                            G_MAXINT,
                                                            3,
-                                                           G_PARAM_READWRITE |
-                                                           G_PARAM_CONSTRUCT_ONLY));
+                                                           PIXBUF_PARAM_FLAGS));
 
         g_object_class_install_property (object_class,
                                          PROP_COLORSPACE,
@@ -112,29 +121,31 @@ gdk_pixbuf_class_init (GdkPixbufClass *klass)
                                                             P_("The colorspace in which the samples are interpreted"),
                                                             GDK_TYPE_COLORSPACE,
                                                             GDK_COLORSPACE_RGB,
-                                                            
-                                                            G_PARAM_READWRITE |
-                                                            G_PARAM_CONSTRUCT_ONLY));
+                                                            PIXBUF_PARAM_FLAGS));
 
         g_object_class_install_property (object_class,
                                          PROP_HAS_ALPHA,
-                                         g_param_spec_boolean ("has_alpha",
+                                         g_param_spec_boolean ("has-alpha",
                                                                P_("Has Alpha"),
                                                                P_("Whether the pixbuf has an alpha channel"),
                                                                FALSE,
-                                                               G_PARAM_READWRITE |
-                                                               G_PARAM_CONSTRUCT_ONLY));
+                                                               PIXBUF_PARAM_FLAGS));
 
+        /**
+         * GdkPixbuf:bits-per-sample:
+         *
+         * The number of bits per sample. 
+         * Currently only 8 bit per sample are supported.
+         */
         g_object_class_install_property (object_class,
                                          PROP_BITS_PER_SAMPLE,
-                                         g_param_spec_int ("bits_per_sample",
+                                         g_param_spec_int ("bits-per-sample",
                                                            P_("Bits per Sample"),
                                                            P_("The number of bits per sample"),
                                                            1,
                                                            16,
                                                            8,
-                                                           G_PARAM_READWRITE |
-                                                           G_PARAM_CONSTRUCT_ONLY));
+                                                           PIXBUF_PARAM_FLAGS));
 
         g_object_class_install_property (object_class,
                                          PROP_WIDTH,
@@ -144,8 +155,7 @@ gdk_pixbuf_class_init (GdkPixbufClass *klass)
                                                            1,
                                                            G_MAXINT,
                                                            1,
-                                                           G_PARAM_READWRITE |
-                                                           G_PARAM_CONSTRUCT_ONLY));
+                                                           PIXBUF_PARAM_FLAGS));
 
         g_object_class_install_property (object_class,
                                          PROP_HEIGHT,
@@ -155,9 +165,15 @@ gdk_pixbuf_class_init (GdkPixbufClass *klass)
                                                            1,
                                                            G_MAXINT,
                                                            1,
-                                                           G_PARAM_READWRITE |
-                                                           G_PARAM_CONSTRUCT_ONLY));
+                                                           PIXBUF_PARAM_FLAGS));
 
+        /**
+         * GdkPixbuf:rowstride:
+         *
+         * The number of bytes between the start of a row and 
+         * the start of the next row. This number must (obviously)
+         * be at least as large as the width of the pixbuf.
+         */
         g_object_class_install_property (object_class,
                                          PROP_ROWSTRIDE,
                                          g_param_spec_int ("rowstride",
@@ -166,16 +182,14 @@ gdk_pixbuf_class_init (GdkPixbufClass *klass)
                                                            1,
                                                            G_MAXINT,
                                                            1,
-                                                           G_PARAM_READWRITE |
-                                                           G_PARAM_CONSTRUCT_ONLY));
+                                                           PIXBUF_PARAM_FLAGS));
 
         g_object_class_install_property (object_class,
                                          PROP_PIXELS,
                                          g_param_spec_pointer ("pixels",
                                                                P_("Pixels"),
                                                                P_("A pointer to the pixel data of the pixbuf"),
-                                                               G_PARAM_READWRITE |
-                                                               G_PARAM_CONSTRUCT_ONLY));
+                                                               PIXBUF_PARAM_FLAGS));
 }
 
 static void
@@ -518,11 +532,11 @@ gdk_pixbuf_get_rowstride (const GdkPixbuf *pixbuf)
 
 
 /* General initialization hooks */
-const guint gdk_pixbuf_major_version = GDK_PIXBUF_MAJOR;
-const guint gdk_pixbuf_minor_version = GDK_PIXBUF_MINOR;
-const guint gdk_pixbuf_micro_version = GDK_PIXBUF_MICRO;
+GDK_PIXBUF_VAR const guint gdk_pixbuf_major_version = GDK_PIXBUF_MAJOR;
+GDK_PIXBUF_VAR const guint gdk_pixbuf_minor_version = GDK_PIXBUF_MINOR;
+GDK_PIXBUF_VAR const guint gdk_pixbuf_micro_version = GDK_PIXBUF_MICRO;
 
-const char *gdk_pixbuf_version = GDK_PIXBUF_VERSION;
+GDK_PIXBUF_VAR const char *gdk_pixbuf_version = GDK_PIXBUF_VERSION;
 
 /* Error quark */
 GQuark
@@ -772,3 +786,6 @@ gdk_pixbuf_get_property (GObject         *object,
 /* Include the marshallers */
 #include <glib-object.h>
 #include "gdk-pixbuf-marshal.c"
+
+#define __GDK_PIXBUF_C__
+#include "gdk-pixbuf-aliasdef.c"
