@@ -27,9 +27,9 @@
 #include <config.h>
 #include <string.h>		/* For memset() */
 
-#include "gdkalias.h"
 #include "gdk.h"
 #include "gdkinternals.h"
+#include "gdkalias.h"
 
 typedef struct _GdkIOClosure GdkIOClosure;
 
@@ -561,6 +561,7 @@ gdk_event_get_time (GdkEvent *event)
       case GDK_WINDOW_STATE:
       case GDK_SETTING:
       case GDK_OWNER_CHANGE:
+      case GDK_GRAB_BROKEN:
         /* return current time */
         break;
       }
@@ -637,6 +638,7 @@ gdk_event_get_state (GdkEvent        *event,
       case GDK_WINDOW_STATE:
       case GDK_SETTING:
       case GDK_OWNER_CHANGE:
+      case GDK_GRAB_BROKEN:
         /* no state field */
         break;
       }
@@ -730,6 +732,10 @@ gdk_event_get_root_coords (GdkEvent *event,
       x = event->motion.x_root;
       y = event->motion.y_root;
       break;
+    case GDK_SCROLL:
+      x = event->scroll.x_root;
+      y = event->scroll.y_root;
+      break;
     case GDK_BUTTON_PRESS:
     case GDK_2BUTTON_PRESS:
     case GDK_3BUTTON_PRESS:
@@ -869,7 +875,7 @@ gdk_event_set_screen (GdkEvent  *event,
  * Returns the screen for the event. The screen is
  * typically the screen for <literal>event->any.window</literal>, but
  * for events such as mouse events, it is the screen
- * where the the pointer was when the event occurs -
+ * where the pointer was when the event occurs -
  * that is, the screen which has the root window 
  * to which <literal>event->motion.x_root</literal> and
  * <literal>event->motion.y_root</literal> are relative.
@@ -1217,3 +1223,6 @@ gdk_setting_get (const gchar *name,
 {
   return gdk_screen_get_setting (gdk_screen_get_default (), name, value);
 }
+
+#define __GDK_EVENTS_C__
+#include "gdkaliasdef.c"
