@@ -37,6 +37,7 @@
 #include "gdkregion-generic.h"
 #include "gdkkeysyms.h"
 #include "gdkinternals.h"
+#include "gdkintl.h"
 #include "gdkprivate-win32.h"
 #include "gdkinput-win32.h"
 
@@ -52,15 +53,20 @@
 
 static gboolean gdk_synchronize = FALSE;
 
-GdkArgDesc _gdk_windowing_args[] = {
-  { "sync",          GDK_ARG_BOOL, &gdk_synchronize, (GdkArgFunc) NULL},
-  { "no-wintab",     GDK_ARG_BOOL, &_gdk_input_ignore_wintab,
-						     (GdkArgFunc) NULL},
-  { "ignore-wintab", GDK_ARG_BOOL, &_gdk_input_ignore_wintab,
-						     (GdkArgFunc) NULL},
-  { "use-wintab",    GDK_ARG_NOBOOL, &_gdk_input_ignore_wintab,
-						     (GdkArgFunc) NULL},
-  { "max-colors",    GDK_ARG_INT,  &_gdk_max_colors,  (GdkArgFunc) NULL},
+static gboolean dummy;
+
+GOptionEntry _gdk_windowing_args[] = {
+  { "sync", 0, 0, G_OPTION_ARG_NONE, &gdk_synchronize, 
+    /* Description of --sync in --help output */              N_("Don't batch GDI requests"), NULL },
+  { "no-wintab", 0, 0, G_OPTION_ARG_NONE, &_gdk_input_ignore_wintab, 
+    /* Description of --no-wintab in --help output */         N_("Don't use the Wintab API for tablet support"), NULL },
+  { "ignore-wintab", 0, 0, G_OPTION_ARG_NONE, &_gdk_input_ignore_wintab, 
+    /* Description of --ignore-wintab in --help output */     N_("Same as --no-wintab"), NULL },
+  { "use-wintab", 0, 0, G_OPTION_ARG_NONE, &dummy,
+    /* Description of --use-wintab in --help output */     N_("Do use the Wintab API [default]"), NULL },
+  { "max-colors", 0, 0, G_OPTION_ARG_INT, &_gdk_max_colors, 
+    /* Description of --max-colors=COLORS in --help output */ N_("Size of the palette in 8 bit mode"), 
+    /* Placeholder in --max-colors=COLORS in --help output */ N_("COLORS") },
   { NULL }
 };
 
@@ -75,8 +81,7 @@ DllMain (HINSTANCE hinstDLL,
 }
 
 void
-_gdk_windowing_init (gint    *argc,
-                     gchar ***argv)
+_gdk_windowing_init (void)
 {
   gchar buf[10];
 
@@ -92,14 +97,6 @@ _gdk_windowing_init (gint    *argc,
 
   _gdk_app_hmodule = GetModuleHandle (NULL);
   _gdk_display_hdc = CreateDC ("DISPLAY", NULL, NULL, NULL);
-  _gdk_root_window = GetDesktopWindow ();
-  _windows_version = GetVersion ();
-
-  if (getenv ("PRETEND_WIN9X"))
-    _windows_version = 0x80000004;
-
-  GDK_NOTE (MISC, g_print ("Windows version: %08x\n", (guint) _windows_version));
-
   _gdk_input_locale = GetKeyboardLayout (0);
   _gdk_input_locale_is_ime = ImmIsIME (_gdk_input_locale);
   GetLocaleInfo (MAKELCID (LOWORD (_gdk_input_locale), SORT_DEFAULT),
@@ -115,8 +112,10 @@ _gdk_windowing_init (gint    *argc,
   _cf_utf8_string = RegisterClipboardFormat ("UTF8_STRING");
 
   _utf8_string = gdk_atom_intern ("UTF8_STRING", FALSE);
-  _compound_text = gdk_atom_intern ("COMPOUND_TEXT", FALSE);
+  _targets = gdk_atom_intern ("TARGETS", FALSE);
+
   _text_uri_list = gdk_atom_intern ("text/uri-list", FALSE);
+  _image_bmp = gdk_atom_intern ("image/bmp", FALSE);
 
   _local_dnd = gdk_atom_intern ("LocalDndSelection", FALSE);
   _gdk_win32_dropfiles = gdk_atom_intern ("DROPFILES_DND", FALSE);
@@ -155,7 +154,7 @@ _gdk_win32_gdi_failed (const gchar *where,
   /* On Win9x GDI calls are implemented in 16-bit code and thus
    * don't set the 32-bit error code, sigh.
    */
-  if (IS_WIN_NT ())
+  if (G_WIN32_IS_NT_BASED ())
     _gdk_win32_api_failed (where, line, api);
   else
     _gdk_other_api_failed (where, line, api);
@@ -176,24 +175,24 @@ gdk_get_use_xshm (void)
 gint
 gdk_screen_get_width (GdkScreen *screen)
 {
-  return GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (_gdk_parent_root)->impl)->width;
+  return GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (_gdk_root)->impl)->width;
 }
 
 gint
 gdk_screen_get_height (GdkScreen *screen)
 {
-  return GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (_gdk_parent_root)->impl)->height;
+  return GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (_gdk_root)->impl)->height;
 }
 gint
 gdk_screen_get_width_mm (GdkScreen *screen)
 {
-  return (double) GetDeviceCaps (_gdk_display_hdc, HORZRES) / GetDeviceCaps (_gdk_display_hdc, LOGPIXELSX) * 25.4;
+  return (double) gdk_screen_get_width (screen) / GetDeviceCaps (_gdk_display_hdc, LOGPIXELSX) * 25.4;
 }
 
 gint
 gdk_screen_get_height_mm (GdkScreen *screen)
 {
-  return (double) GetDeviceCaps (_gdk_display_hdc, VERTRES) / GetDeviceCaps (_gdk_display_hdc, LOGPIXELSY) * 25.4;
+  return (double) gdk_screen_get_height (screen) / GetDeviceCaps (_gdk_display_hdc, LOGPIXELSY) * 25.4;
 }
 
 void
@@ -253,7 +252,7 @@ gdk_notify_startup_complete (void)
  */
 static gchar *
 static_printf (const gchar *format,
-			 ...)
+	       ...)
 {
   static gchar buf[10000];
   gchar *msg;
@@ -622,14 +621,15 @@ _gdk_win32_psstyle_to_string (DWORD pen_style)
   switch (pen_style & PS_STYLE_MASK)
     {
 #define CASE(x) case PS_##x: return #x
+      CASE (ALTERNATE);
+      CASE (SOLID);
       CASE (DASH);
+      CASE (DOT);
       CASE (DASHDOT);
       CASE (DASHDOTDOT);
-      CASE (DOT);
-      CASE (INSIDEFRAME);
       CASE (NULL);
-      CASE (SOLID);
       CASE (USERSTYLE);
+      CASE (INSIDEFRAME);
 #undef CASE
     default: return static_printf ("illegal_%d", pen_style & PS_STYLE_MASK);
     }
@@ -643,9 +643,9 @@ _gdk_win32_psendcap_to_string (DWORD pen_style)
   switch (pen_style & PS_ENDCAP_MASK)
     {
 #define CASE(x) case PS_ENDCAP_##x: return #x
-      CASE (FLAT);
       CASE (ROUND);
       CASE (SQUARE);
+      CASE (FLAT);
 #undef CASE
     default: return static_printf ("illegal_%d", pen_style & PS_ENDCAP_MASK);
     }
@@ -659,9 +659,9 @@ _gdk_win32_psjoin_to_string (DWORD pen_style)
   switch (pen_style & PS_JOIN_MASK)
     {
 #define CASE(x) case PS_JOIN_##x: return #x
+      CASE (ROUND);
       CASE (BEVEL);
       CASE (MITER);
-      CASE (ROUND);
 #undef CASE
     default: return static_printf ("illegal_%d", pen_style & PS_JOIN_MASK);
     }
@@ -906,10 +906,95 @@ _gdk_win32_message_to_string (UINT msg)
   return NULL;
 }
 
+gchar *
+_gdk_win32_key_to_string (LONG lParam)
+{
+  char buf[100];
+  gchar *keyname_utf8;
+
+  if (GetKeyNameText (lParam, buf, sizeof (buf)) &&
+      (keyname_utf8 = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL)) != NULL)
+    {
+      gchar *retval = static_printf ("%s", keyname_utf8);
+
+      g_free (keyname_utf8);
+
+      return retval;
+    }
+
+  return static_printf ("unk-%#lx", lParam);
+}
+      
+gchar *
+_gdk_win32_cf_to_string (UINT format)
+{
+  char buf[100];
+
+  switch (format)
+    {
+#define CASE(x) case CF_##x: return #x
+      CASE (BITMAP);
+      CASE (DIB);
+#ifdef CF_DIBV5
+      CASE (DIBV5);
+#endif
+      CASE (DIF);
+      CASE (DSPBITMAP);
+      CASE (DSPENHMETAFILE);
+      CASE (DSPMETAFILEPICT);
+      CASE (DSPTEXT);
+      CASE (ENHMETAFILE);
+      CASE (HDROP);
+      CASE (LOCALE);
+      CASE (METAFILEPICT);
+      CASE (OEMTEXT);
+      CASE (OWNERDISPLAY);
+      CASE (PALETTE);
+      CASE (PENDATA);
+      CASE (RIFF);
+      CASE (SYLK);
+      CASE (TEXT);
+      CASE (WAVE);
+      CASE (TIFF);
+      CASE (UNICODETEXT);
+    default:
+      if (format >= CF_GDIOBJFIRST &&
+	  format <= CF_GDIOBJLAST)
+	return static_printf ("CF_GDIOBJ%d", format - CF_GDIOBJFIRST);
+      if (format >= CF_PRIVATEFIRST &&
+	  format <= CF_PRIVATELAST)
+	return static_printf ("CF_PRIVATE%d", format - CF_PRIVATEFIRST);
+      if (GetClipboardFormatName (format, buf, sizeof (buf)))
+	return static_printf ("%s", buf);
+      else
+	return static_printf ("unk-%#lx", format);
+    }
+}
+      
+gchar *
+_gdk_win32_data_to_string (const guchar *data,
+			   int           nbytes)
+{
+  GString *s = g_string_new ("");
+  int i;
+  gchar *retval;
+
+  for (i = 0; i < nbytes; i++)
+    if (data[i] >=' ' && data[i] <= '~')
+      g_string_append_printf (s, "%c  ", data[i]);
+    else
+      g_string_append_printf (s, "%02X ", data[i]);
+
+  retval = static_printf ("%s", s->str);
+  g_string_free (s, TRUE);
+
+  return retval;
+}
+
 gchar *
 _gdk_win32_rect_to_string (const RECT *rect)
 {
-  return static_printf ("%ldx%ld@+%ld+%ld",
+  return static_printf ("%ldx%ld@%+ld%+ld",
 			(rect->right - rect->left), (rect->bottom - rect->top),
 			rect->left, rect->top);
 }
@@ -917,7 +1002,7 @@ _gdk_win32_rect_to_string (const RECT *rect)
 gchar *
 _gdk_win32_gdkrectangle_to_string (const GdkRectangle *rect)
 {
-  return static_printf ("%dx%d@+%d+%d",
+  return static_printf ("%dx%d@%+d%+d",
 			rect->width, rect->height,
 			rect->x, rect->y);
 }
@@ -925,7 +1010,7 @@ _gdk_win32_gdkrectangle_to_string (const GdkRectangle *rect)
 gchar *
 _gdk_win32_gdkregion_to_string (const GdkRegion *rgn)
 {
-  return static_printf ("%dx%d@+%d+%d",
+  return static_printf ("%dx%d@%+d%+d",
 			(rgn->extents.x2 - rgn->extents.x1),
 			(rgn->extents.y2 - rgn->extents.y1),
 			rgn->extents.x1, rgn->extents.y1);
