@@ -1849,7 +1849,7 @@ gdk_window_set_back_pixmap (GdkWindow *window,
   g_return_if_fail (pixmap == NULL || !parent_relative);
   g_return_if_fail (pixmap == NULL || gdk_drawable_get_depth (window) == gdk_drawable_get_depth (pixmap));
   
-  if (!gdk_drawable_get_colormap (pixmap))
+  if (pixmap && !gdk_drawable_get_colormap (pixmap))
     {
       g_warning ("gdk_window_set_back_pixmap(): pixmap must have a colormap");
       return;
