@@ -36,6 +36,7 @@
 #include "gtkradioaction.h"
 #include "gtkaccelmap.h"
 #include "gtkmarshalers.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkalias.h"
 
@@ -134,22 +135,21 @@ gtk_action_group_class_init (GtkActionGroupClass *klass)
 							P_("Name"),
 							P_("A name for the action group."),
 							NULL,
-							G_PARAM_READWRITE |
-							G_PARAM_CONSTRUCT_ONLY));
+							GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   g_object_class_install_property (gobject_class,
 				   PROP_SENSITIVE,
 				   g_param_spec_boolean ("sensitive",
 							 P_("Sensitive"),
 							 P_("Whether the action group is enabled."),
 							 TRUE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
 				   PROP_VISIBLE,
 				   g_param_spec_boolean ("visible",
 							 P_("Visible"),
 							 P_("Whether the action group is visible."),
 							 TRUE,
-							 G_PARAM_READWRITE));
+							 GTK_PARAM_READWRITE));
 
   /**
    * GtkActionGroup::connect-proxy:
@@ -251,7 +251,7 @@ gtk_action_group_class_init (GtkActionGroupClass *klass)
 static void 
 remove_action (GtkAction *action) 
 {
-  g_object_set (action, "action_group", NULL, NULL);
+  g_object_set (action, "action-group", NULL, NULL);
   g_object_unref (action);
 }
 
@@ -419,7 +419,7 @@ gtk_action_group_get_sensitive (GtkActionGroup *action_group)
 static void
 cb_set_action_sensitivity (const gchar *name, GtkAction *action)
 {
-  /* Minor optimization, the action_groups state only effects actions that are
+  /* Minor optimization, the action_groups state only affects actions that are
    * themselves sensitive */
   if (gtk_action_get_sensitive (action))
     g_object_notify (G_OBJECT (action), "sensitive");
@@ -471,7 +471,7 @@ gtk_action_group_get_visible (GtkActionGroup *action_group)
 static void
 cb_set_action_visiblity (const gchar *name, GtkAction *action)
 {
-  /* Minor optimization, the action_groups state only effects actions that are
+  /* Minor optimization, the action_groups state only affects actions that are
    * themselves sensitive */
   if (gtk_action_get_visible (action))
     g_object_notify (G_OBJECT (action), "visible");
@@ -546,7 +546,7 @@ gtk_action_group_add_action (GtkActionGroup *action_group,
   g_hash_table_insert (action_group->private_data->actions, 
 		       g_strdup (gtk_action_get_name (action)),
                        g_object_ref (action));
-  g_object_set (action, "action_group", action_group, NULL);
+  g_object_set (action, "action-group", action_group, NULL);
 }
 
 /**
@@ -579,7 +579,7 @@ gtk_action_group_add_action_with_accel (GtkActionGroup *action_group,
   gchar *name;
   gchar *stock_id;
   
-  g_object_get (action, "name", &name, "stock_id", &stock_id, NULL);
+  g_object_get (action, "name", &name, "stock-id", &stock_id, NULL);
 
   accel_path = g_strconcat ("<Actions>/",
 			    action_group->private_data->name, "/", name, NULL);
@@ -603,7 +603,7 @@ gtk_action_group_add_action_with_accel (GtkActionGroup *action_group,
     }
 
   if (accel_key)
-    gtk_accel_map_change_entry (accel_path, accel_key, accel_mods, TRUE);
+    gtk_accel_map_add_entry (accel_path, accel_key, accel_mods);
 
   gtk_action_set_accel_path (action, accel_path);
   gtk_action_group_add_action (action_group, action);
