@@ -19,13 +19,14 @@
 
 #include <config.h>
 #include <stdlib.h>
-#include "gtkalias.h"
 #include "gtkcellrenderertext.h"
 #include "gtkeditable.h"
 #include "gtkentry.h"
 #include "gtkmarshalers.h"
 #include "gtkintl.h"
+#include "gtkprivate.h"
 #include "gtktreeprivate.h"
+#include "gtkalias.h"
 
 static void gtk_cell_renderer_text_init       (GtkCellRendererText      *celltext);
 static void gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class);
@@ -74,6 +75,8 @@ enum {
   PROP_MARKUP,
   PROP_ATTRIBUTES,
   PROP_SINGLE_PARAGRAPH_MODE,
+  PROP_WIDTH_CHARS,
+  PROP_WRAP_WIDTH,
   
   /* Style args */
   PROP_BACKGROUND,
@@ -96,6 +99,7 @@ enum {
   PROP_RISE,
   PROP_LANGUAGE,
   PROP_ELLIPSIZE,
+  PROP_WRAP_MODE,
   
   /* Whether-a-style-arg-is-set args */
   PROP_BACKGROUND_SET,
@@ -133,11 +137,15 @@ struct _GtkCellRendererTextPrivate
   gulong focus_out_id;
   PangoLanguage *language;
   PangoEllipsizeMode ellipsize;
+  PangoWrapMode wrap_mode;
   
   gulong populate_popup_id;
   gulong entry_menu_popdown_timeout;
   gboolean in_entry_menu;
-
+  
+  gint width_chars;
+  gint wrap_width;
+  
   GtkWidget *entry;
 };
 
@@ -173,12 +181,19 @@ gtk_cell_renderer_text_get_type (void)
 static void
 gtk_cell_renderer_text_init (GtkCellRendererText *celltext)
 {
+  GtkCellRendererTextPrivate *priv;
+
+  priv = GTK_CELL_RENDERER_TEXT_GET_PRIVATE (celltext);
+
   GTK_CELL_RENDERER (celltext)->xalign = 0.0;
   GTK_CELL_RENDERER (celltext)->yalign = 0.5;
   GTK_CELL_RENDERER (celltext)->xpad = 2;
   GTK_CELL_RENDERER (celltext)->ypad = 2;
   celltext->fixed_height_rows = -1;
   celltext->font = pango_font_description_new ();
+
+  priv->width_chars = -1;
+  priv->wrap_width = -1;
 }
 
 static void
@@ -197,14 +212,14 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
   cell_class->get_size = gtk_cell_renderer_text_get_size;
   cell_class->render = gtk_cell_renderer_text_render;
   cell_class->start_editing = gtk_cell_renderer_text_start_editing;
-  
+
   g_object_class_install_property (object_class,
                                    PROP_TEXT,
                                    g_param_spec_string ("text",
                                                         P_("Text"),
                                                         P_("Text to render"),
                                                         NULL,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
   
   g_object_class_install_property (object_class,
                                    PROP_MARKUP,
@@ -212,7 +227,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                         P_("Markup"),
                                                         P_("Marked up text to render"),
                                                         NULL,
-                                                        G_PARAM_WRITABLE));
+                                                        GTK_PARAM_WRITABLE));
 
   g_object_class_install_property (object_class,
 				   PROP_ATTRIBUTES,
@@ -220,15 +235,15 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
 						       P_("Attributes"),
 						       P_("A list of style attributes to apply to the text of the renderer"),
 						       PANGO_TYPE_ATTR_LIST,
-						       G_PARAM_READWRITE));
+						       GTK_PARAM_READWRITE));
 
   g_object_class_install_property (object_class,
                                    PROP_SINGLE_PARAGRAPH_MODE,
-                                   g_param_spec_boolean ("single_paragraph_mode",
+                                   g_param_spec_boolean ("single-paragraph-mode",
                                                          P_("Single Paragraph Mode"),
                                                          P_("Whether or not to keep all text in a single paragraph"),
                                                          FALSE,
-                                                         G_PARAM_READWRITE));
+                                                         GTK_PARAM_READWRITE));
 
   
   g_object_class_install_property (object_class,
@@ -237,15 +252,15 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                         P_("Background color name"),
                                                         P_("Background color as a string"),
                                                         NULL,
-                                                        G_PARAM_WRITABLE));
+                                                        GTK_PARAM_WRITABLE));
 
   g_object_class_install_property (object_class,
                                    PROP_BACKGROUND_GDK,
-                                   g_param_spec_boxed ("background_gdk",
+                                   g_param_spec_boxed ("background-gdk",
                                                        P_("Background color"),
                                                        P_("Background color as a GdkColor"),
                                                        GDK_TYPE_COLOR,
-                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));  
+                                                       GTK_PARAM_READWRITE));  
 
   g_object_class_install_property (object_class,
                                    PROP_FOREGROUND,
@@ -253,15 +268,15 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                         P_("Foreground color name"),
                                                         P_("Foreground color as a string"),
                                                         NULL,
-                                                        G_PARAM_WRITABLE));
+                                                        GTK_PARAM_WRITABLE));
 
   g_object_class_install_property (object_class,
                                    PROP_FOREGROUND_GDK,
-                                   g_param_spec_boxed ("foreground_gdk",
+                                   g_param_spec_boxed ("foreground-gdk",
                                                        P_("Foreground color"),
                                                        P_("Foreground color as a GdkColor"),
                                                        GDK_TYPE_COLOR,
-                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                       GTK_PARAM_READWRITE));
 
 
   g_object_class_install_property (object_class,
@@ -270,7 +285,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                          P_("Editable"),
                                                          P_("Whether the text can be modified by the user"),
                                                          FALSE,
-                                                         G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                         GTK_PARAM_READWRITE));
 
   g_object_class_install_property (object_class,
                                    PROP_FONT,
@@ -278,15 +293,15 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                         P_("Font"),
                                                         P_("Font description as a string"),
                                                         NULL,
-                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (object_class,
                                    PROP_FONT_DESC,
-                                   g_param_spec_boxed ("font_desc",
+                                   g_param_spec_boxed ("font-desc",
                                                        P_("Font"),
                                                        P_("Font description as a PangoFontDescription struct"),
                                                        PANGO_TYPE_FONT_DESCRIPTION,
-                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                       GTK_PARAM_READWRITE));
 
   
   g_object_class_install_property (object_class,
@@ -295,7 +310,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                         P_("Font family"),
                                                         P_("Name of the font family, e.g. Sans, Helvetica, Times, Monospace"),
                                                         NULL,
-                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (object_class,
                                    PROP_STYLE,
@@ -304,7 +319,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                       P_("Font style"),
                                                       PANGO_TYPE_STYLE,
                                                       PANGO_STYLE_NORMAL,
-                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (object_class,
                                    PROP_VARIANT,
@@ -313,7 +328,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                      P_("Font variant"),
                                                       PANGO_TYPE_VARIANT,
                                                       PANGO_VARIANT_NORMAL,
-                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                      GTK_PARAM_READWRITE));
   
   g_object_class_install_property (object_class,
                                    PROP_WEIGHT,
@@ -323,7 +338,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                      0,
                                                      G_MAXINT,
                                                      PANGO_WEIGHT_NORMAL,
-                                                     G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                     GTK_PARAM_READWRITE));
 
    g_object_class_install_property (object_class,
                                    PROP_STRETCH,
@@ -332,7 +347,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                       P_("Font stretch"),
                                                       PANGO_TYPE_STRETCH,
                                                       PANGO_STRETCH_NORMAL,
-                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                      GTK_PARAM_READWRITE));
   
   g_object_class_install_property (object_class,
                                    PROP_SIZE,
@@ -342,17 +357,17 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                      0,
                                                      G_MAXINT,
                                                      0,
-                                                     G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                     GTK_PARAM_READWRITE));
 
   g_object_class_install_property (object_class,
                                    PROP_SIZE_POINTS,
-                                   g_param_spec_double ("size_points",
+                                   g_param_spec_double ("size-points",
                                                         P_("Font points"),
                                                         P_("Font size in points"),
                                                         0.0,
                                                         G_MAXDOUBLE,
                                                         0.0,
-                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));  
+                                                        GTK_PARAM_READWRITE));  
 
   g_object_class_install_property (object_class,
                                    PROP_SCALE,
@@ -362,17 +377,18 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                         0.0,
                                                         G_MAXDOUBLE,
                                                         1.0,
-                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                        GTK_PARAM_READWRITE));
   
   g_object_class_install_property (object_class,
                                    PROP_RISE,
                                    g_param_spec_int ("rise",
                                                      P_("Rise"),
-                                                     P_("Offset of text above the baseline (below the baseline if rise is negative)"),
+                                                     P_("Offset of text above the baseline "
+							"(below the baseline if rise is negative)"),
                                                      -G_MAXINT,
                                                      G_MAXINT,
                                                      0,
-                                                     G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                     GTK_PARAM_READWRITE));
 
 
   g_object_class_install_property (object_class,
@@ -381,7 +397,7 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                          P_("Strikethrough"),
                                                          P_("Whether to strike through the text"),
                                                          FALSE,
-                                                         G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                         GTK_PARAM_READWRITE));
   
   g_object_class_install_property (object_class,
                                    PROP_UNDERLINE,
@@ -390,90 +406,163 @@ gtk_cell_renderer_text_class_init (GtkCellRendererTextClass *class)
                                                       P_("Style of underline for this text"),
                                                       PANGO_TYPE_UNDERLINE,
                                                       PANGO_UNDERLINE_NONE,
-                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (object_class,
                                    PROP_LANGUAGE,
                                    g_param_spec_string ("language",
                                                         P_("Language"),
-                                                        P_("The language this text is in, as an ISO code. Pango can use this as a hint when rendering the text. If you don't understand this parameter, you probably don't need it"),
+                                                        P_("The language this text is in, as an ISO code. "
+							   "Pango can use this as a hint when rendering the text. "
+							   "If you don't understand this parameter, you probably don't need it"),
                                                         NULL,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
 
+  /**
+   * GtkCellRendererText:ellipsize:
+   *
+   * Specifies the preferred place to ellipsize the string, if the cell renderer 
+   * does not have enough room to display the entire string. Setting it to 
+   * %PANGO_ELLIPSIZE_NONE turns off ellipsizing. See the wrap-width property
+   * for another way of making the text fit into a given width.
+   *
+   * Since: 2.6
+   */
   g_object_class_install_property (object_class,
                                    PROP_ELLIPSIZE,
                                    g_param_spec_enum ("ellipsize",
 						      P_("Ellipsize"),
-						      P_("The preferred place to ellipsize the string, if the cell renderer does not have enough room to display the entire string, if at all"),							
+						      P_("The preferred place to ellipsize the string, "
+							 "if the cell renderer does not have enough room "
+							 "to display the entire string, if at all"),
 						      PANGO_TYPE_ELLIPSIZE_MODE,
 						      PANGO_ELLIPSIZE_NONE,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
+
+  /**
+   * GtkCellRendererText:width-chars:
+   * 
+   * The desired width of the cell, in characters. If this property is set to
+   * -1, the width will be calculated automatically, otherwise the cell will
+   * request either 3 characters or the property value, whichever is greater.
+   * 
+   * Since: 2.6
+   **/
+  g_object_class_install_property (object_class,
+                                   PROP_WIDTH_CHARS,
+                                   g_param_spec_int ("width-chars",
+                                                     P_("Width In Characters"),
+                                                     P_("The desired width of the label, in characters"),
+                                                     -1,
+                                                     G_MAXINT,
+                                                     -1,
+                                                     GTK_PARAM_READWRITE));
+  
+  /**
+   * GtkCellRendererText:wrap-mode:
+   *
+   * Specifies how to break the string into multiple lines, if the cell 
+   * renderer does not have enough room to display the entire string. 
+   * This property has no effect unless the wrap-width property is set.
+   *
+   * Since: 2.8
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_WRAP_MODE,
+                                   g_param_spec_enum ("wrap-mode",
+						      P_("Wrap mode"),
+						      P_("How to break the string into multiple lines, "
+							 "if the cell renderer does not have enough room "
+							 "to display the entire string"),
+						      PANGO_TYPE_WRAP_MODE,
+						      PANGO_WRAP_CHAR,
+						      GTK_PARAM_READWRITE));
+
+  /**
+   * GtkCellRendererText:wrap-width:
+   *
+   * Specifies the width at which the text is wrapped. The wrap-mode property can 
+   * be used to influence at what character positions the line breaks can be placed.
+   * Setting wrap-width to -1 turns wrapping off.
+   *
+   * Since: 2.8
+   */
+  g_object_class_install_property (object_class,
+				   PROP_WRAP_WIDTH,
+				   g_param_spec_int ("wrap-width",
+						     P_("Wrap width"),
+						     P_("The width at which the text is wrapped"),
+						     -1,
+						     G_MAXINT,
+						     -1,
+						     GTK_PARAM_READWRITE));
+
   
   /* Style props are set or not */
 
-#define ADD_SET_PROP(propname, propval, nick, blurb) g_object_class_install_property (object_class, propval, g_param_spec_boolean (propname, nick, blurb, FALSE, G_PARAM_READABLE | G_PARAM_WRITABLE))
+#define ADD_SET_PROP(propname, propval, nick, blurb) g_object_class_install_property (object_class, propval, g_param_spec_boolean (propname, nick, blurb, FALSE, GTK_PARAM_READWRITE))
 
-  ADD_SET_PROP ("background_set", PROP_BACKGROUND_SET,
+  ADD_SET_PROP ("background-set", PROP_BACKGROUND_SET,
                 P_("Background set"),
                 P_("Whether this tag affects the background color"));
 
-  ADD_SET_PROP ("foreground_set", PROP_FOREGROUND_SET,
+  ADD_SET_PROP ("foreground-set", PROP_FOREGROUND_SET,
                 P_("Foreground set"),
                 P_("Whether this tag affects the foreground color"));
   
-  ADD_SET_PROP ("editable_set", PROP_EDITABLE_SET,
+  ADD_SET_PROP ("editable-set", PROP_EDITABLE_SET,
                 P_("Editability set"),
                 P_("Whether this tag affects text editability"));
 
-  ADD_SET_PROP ("family_set", PROP_FAMILY_SET,
+  ADD_SET_PROP ("family-set", PROP_FAMILY_SET,
                 P_("Font family set"),
                 P_("Whether this tag affects the font family"));  
 
-  ADD_SET_PROP ("style_set", PROP_STYLE_SET,
+  ADD_SET_PROP ("style-set", PROP_STYLE_SET,
                 P_("Font style set"),
                 P_("Whether this tag affects the font style"));
 
-  ADD_SET_PROP ("variant_set", PROP_VARIANT_SET,
+  ADD_SET_PROP ("variant-set", PROP_VARIANT_SET,
                 P_("Font variant set"),
                 P_("Whether this tag affects the font variant"));
 
-  ADD_SET_PROP ("weight_set", PROP_WEIGHT_SET,
+  ADD_SET_PROP ("weight-set", PROP_WEIGHT_SET,
                 P_("Font weight set"),
                 P_("Whether this tag affects the font weight"));
 
-  ADD_SET_PROP ("stretch_set", PROP_STRETCH_SET,
+  ADD_SET_PROP ("stretch-set", PROP_STRETCH_SET,
                 P_("Font stretch set"),
                 P_("Whether this tag affects the font stretch"));
 
-  ADD_SET_PROP ("size_set", PROP_SIZE_SET,
+  ADD_SET_PROP ("size-set", PROP_SIZE_SET,
                 P_("Font size set"),
                 P_("Whether this tag affects the font size"));
 
-  ADD_SET_PROP ("scale_set", PROP_SCALE_SET,
+  ADD_SET_PROP ("scale-set", PROP_SCALE_SET,
                 P_("Font scale set"),
                 P_("Whether this tag scales the font size by a factor"));
   
-  ADD_SET_PROP ("rise_set", PROP_RISE_SET,
+  ADD_SET_PROP ("rise-set", PROP_RISE_SET,
                 P_("Rise set"),
                 P_("Whether this tag affects the rise"));
 
-  ADD_SET_PROP ("strikethrough_set", PROP_STRIKETHROUGH_SET,
+  ADD_SET_PROP ("strikethrough-set", PROP_STRIKETHROUGH_SET,
                 P_("Strikethrough set"),
                 P_("Whether this tag affects strikethrough"));
 
-  ADD_SET_PROP ("underline_set", PROP_UNDERLINE_SET,
+  ADD_SET_PROP ("underline-set", PROP_UNDERLINE_SET,
                 P_("Underline set"),
                 P_("Whether this tag affects underlining"));
 
-  ADD_SET_PROP ("language_set", PROP_LANGUAGE_SET,
+  ADD_SET_PROP ("language-set", PROP_LANGUAGE_SET,
                 P_("Language set"),
                 P_("Whether this tag affects the language the text is rendered as"));
 
-  ADD_SET_PROP ("ellipsize_set", PROP_ELLIPSIZE_SET,
+  ADD_SET_PROP ("ellipsize-set", PROP_ELLIPSIZE_SET,
                 P_("Ellipsize set"),
                 P_("Whether this tag affects the ellipsize mode"));
-  
+
   text_cell_renderer_signals [EDITED] =
     g_signal_new ("edited",
 		  G_OBJECT_CLASS_TYPE (object_class),
@@ -652,6 +741,14 @@ gtk_cell_renderer_text_get_property (GObject        *object,
       g_value_set_enum (value, priv->ellipsize);
       break;
       
+    case PROP_WRAP_MODE:
+      g_value_set_enum (value, priv->wrap_mode);
+      break;
+
+    case PROP_WRAP_WIDTH:
+      g_value_set_int (value, priv->wrap_width);
+      break;
+      
     case PROP_BACKGROUND_SET:
       g_value_set_boolean (value, celltext->background_set);
       break;
@@ -701,6 +798,10 @@ gtk_cell_renderer_text_get_property (GObject        *object,
       g_value_set_boolean (value, priv->ellipsize_set);
       break;
       
+    case PROP_WIDTH_CHARS:
+      g_value_set_int (value, priv->width_chars);
+      break;  
+
     case PROP_BACKGROUND:
     case PROP_FOREGROUND:
     case PROP_MARKUP:
@@ -720,7 +821,7 @@ set_bg_color (GtkCellRendererText *celltext,
       if (!celltext->background_set)
         {
           celltext->background_set = TRUE;
-          g_object_notify (G_OBJECT (celltext), "background_set");
+          g_object_notify (G_OBJECT (celltext), "background-set");
         }
       
       celltext->background.red = color->red;
@@ -732,7 +833,7 @@ set_bg_color (GtkCellRendererText *celltext,
       if (celltext->background_set)
         {
           celltext->background_set = FALSE;
-          g_object_notify (G_OBJECT (celltext), "background_set");
+          g_object_notify (G_OBJECT (celltext), "background-set");
         }
     }
 }
@@ -747,7 +848,7 @@ set_fg_color (GtkCellRendererText *celltext,
       if (!celltext->foreground_set)
         {
           celltext->foreground_set = TRUE;
-          g_object_notify (G_OBJECT (celltext), "foreground_set");
+          g_object_notify (G_OBJECT (celltext), "foreground-set");
         }
       
       celltext->foreground.red = color->red;
@@ -759,7 +860,7 @@ set_fg_color (GtkCellRendererText *celltext,
       if (celltext->foreground_set)
         {
           celltext->foreground_set = FALSE;
-          g_object_notify (G_OBJECT (celltext), "foreground_set");
+          g_object_notify (G_OBJECT (celltext), "foreground-set");
         }
     }
 }
@@ -809,17 +910,17 @@ notify_set_changed (GObject       *object,
 		    PangoFontMask  changed_mask)
 {
   if (changed_mask & PANGO_FONT_MASK_FAMILY)
-    g_object_notify (object, "family_set");
+    g_object_notify (object, "family-set");
   if (changed_mask & PANGO_FONT_MASK_STYLE)
-    g_object_notify (object, "style_set");
+    g_object_notify (object, "style-set");
   if (changed_mask & PANGO_FONT_MASK_VARIANT)
-    g_object_notify (object, "variant_set");
+    g_object_notify (object, "variant-set");
   if (changed_mask & PANGO_FONT_MASK_WEIGHT)
-    g_object_notify (object, "weight_set");
+    g_object_notify (object, "weight-set");
   if (changed_mask & PANGO_FONT_MASK_STRETCH)
-    g_object_notify (object, "stretch_set");
+    g_object_notify (object, "stretch-set");
   if (changed_mask & PANGO_FONT_MASK_SIZE)
-    g_object_notify (object, "size_set");
+    g_object_notify (object, "size-set");
 }
 
 static void
@@ -864,7 +965,7 @@ set_font_description (GtkCellRendererText  *celltext,
   
   g_object_freeze_notify (object);
 
-  g_object_notify (object, "font_desc");
+  g_object_notify (object, "font-desc");
   g_object_notify (object, "font");
   
   if (changed_mask & PANGO_FONT_MASK_FAMILY)
@@ -880,7 +981,7 @@ set_font_description (GtkCellRendererText  *celltext,
   if (changed_mask & PANGO_FONT_MASK_SIZE)
     {
       g_object_notify (object, "size");
-      g_object_notify (object, "size_points");
+      g_object_notify (object, "size-points");
     }
 
   notify_set_changed (object, set_changed_mask);
@@ -974,7 +1075,7 @@ gtk_cell_renderer_text_set_property (GObject      *object,
         else
           g_warning ("Don't know color `%s'", g_value_get_string (value));
 
-        g_object_notify (object, "background_gdk");
+        g_object_notify (object, "background-gdk");
       }
       break;
       
@@ -989,7 +1090,7 @@ gtk_cell_renderer_text_set_property (GObject      *object,
         else
           g_warning ("Don't know color `%s'", g_value_get_string (value));
 
-        g_object_notify (object, "foreground_gdk");
+        g_object_notify (object, "foreground-gdk");
       }
       break;
 
@@ -1064,7 +1165,7 @@ gtk_cell_renderer_text_set_property (GObject      *object,
 	  case PROP_SIZE:
 	    pango_font_description_set_size (celltext->font,
 					     g_value_get_int (value));
-	    g_object_notify (object, "size_points");
+	    g_object_notify (object, "size-points");
 	    break;
 	  case PROP_SIZE_POINTS:
 	    pango_font_description_set_size (celltext->font,
@@ -1077,7 +1178,7 @@ gtk_cell_renderer_text_set_property (GObject      *object,
 	  celltext->calc_fixed_height = TRUE;
 	
 	notify_set_changed (object, old_set_mask & pango_font_description_get_set_fields (celltext->font));
-	g_object_notify (object, "font_desc");
+	g_object_notify (object, "font-desc");
 	g_object_notify (object, "font");
 
 	break;
@@ -1088,7 +1189,7 @@ gtk_cell_renderer_text_set_property (GObject      *object,
       celltext->scale_set = TRUE;
       if (celltext->fixed_height_rows != -1)
 	celltext->calc_fixed_height = TRUE;
-      g_object_notify (object, "scale_set");
+      g_object_notify (object, "scale-set");
       break;
       
     case PROP_EDITABLE:
@@ -1098,26 +1199,26 @@ gtk_cell_renderer_text_set_property (GObject      *object,
         GTK_CELL_RENDERER (celltext)->mode = GTK_CELL_RENDERER_MODE_EDITABLE;
       else
         GTK_CELL_RENDERER (celltext)->mode = GTK_CELL_RENDERER_MODE_INERT;
-      g_object_notify (object, "editable_set");
+      g_object_notify (object, "editable-set");
       break;
 
     case PROP_STRIKETHROUGH:
       celltext->strikethrough = g_value_get_boolean (value);
       celltext->strikethrough_set = TRUE;
-      g_object_notify (object, "strikethrough_set");
+      g_object_notify (object, "strikethrough-set");
       break;
 
     case PROP_UNDERLINE:
       celltext->underline_style = g_value_get_enum (value);
       celltext->underline_set = TRUE;
-      g_object_notify (object, "underline_set");
+      g_object_notify (object, "underline-set");
             
       break;
 
     case PROP_RISE:
       celltext->rise = g_value_get_int (value);
       celltext->rise_set = TRUE;
-      g_object_notify (object, "rise_set");
+      g_object_notify (object, "rise-set");
       if (celltext->fixed_height_rows != -1)
 	celltext->calc_fixed_height = TRUE;
       break;  
@@ -1127,15 +1228,27 @@ gtk_cell_renderer_text_set_property (GObject      *object,
       if (priv->language)
         g_object_unref (priv->language);
       priv->language = pango_language_from_string (g_value_get_string (value));
-      g_object_notify (object, "language_set");
+      g_object_notify (object, "language-set");
       break;
 
     case PROP_ELLIPSIZE:
       priv->ellipsize = g_value_get_enum (value);
       priv->ellipsize_set = TRUE;
-      g_object_notify (object, "ellipsize_set");
+      g_object_notify (object, "ellipsize-set");
       break;
       
+    case PROP_WRAP_MODE:
+      priv->wrap_mode = g_value_get_enum (value);
+      break;
+      
+    case PROP_WRAP_WIDTH:
+      priv->wrap_width = g_value_get_int (value);
+      break;
+            
+    case PROP_WIDTH_CHARS:
+      priv->width_chars = g_value_get_int (value);
+      break;  
+
     case PROP_BACKGROUND_SET:
       celltext->background_set = g_value_get_boolean (value);
       break;
@@ -1190,7 +1303,7 @@ gtk_cell_renderer_text_set_property (GObject      *object,
       break;
 
     case PROP_ELLIPSIZE_SET:
-      priv->ellipsize_set = g_value_get_enum (value);
+      priv->ellipsize_set = g_value_get_boolean (value);
       break;
       
     default:
@@ -1258,7 +1371,8 @@ get_layout (GtkCellRendererText *celltext,
        * background_area not the PangoLayout area
        */
       
-      if (celltext->foreground_set)
+      if (celltext->foreground_set
+	  && (flags & GTK_CELL_RENDERER_SELECTED) == 0)
         {
           PangoColor color;
 
@@ -1314,9 +1428,25 @@ get_layout (GtkCellRendererText *celltext,
     pango_layout_set_ellipsize (layout, priv->ellipsize);
   else
     pango_layout_set_ellipsize (layout, PANGO_ELLIPSIZE_NONE);
-  
+
+  if (priv->wrap_width != -1)
+    {
+      pango_layout_set_width (layout, priv->wrap_width * PANGO_SCALE);
+      pango_layout_set_wrap (layout, priv->wrap_mode);
+
+      if (pango_layout_get_line_count (layout) == 1)
+	{
+	  pango_layout_set_width (layout, -1);
+	  pango_layout_set_wrap (layout, PANGO_WRAP_CHAR);
+	}
+    }
+  else
+    {
+      pango_layout_set_width (layout, -1);
+      pango_layout_set_wrap (layout, PANGO_WRAP_CHAR);
+    }
+
   pango_layout_set_attributes (layout, attr_list);
-  pango_layout_set_width (layout, -1);
 
   pango_attr_list_unref (attr_list);
   
@@ -1324,17 +1454,17 @@ get_layout (GtkCellRendererText *celltext,
 }
 
 static void
-gtk_cell_renderer_text_get_size (GtkCellRenderer *cell,
-				 GtkWidget       *widget,
-				 GdkRectangle    *cell_area,
-				 gint            *x_offset,
-				 gint            *y_offset,
-				 gint            *width,
-				 gint            *height)
+get_size (GtkCellRenderer *cell,
+	  GtkWidget       *widget,
+	  GdkRectangle    *cell_area,
+	  PangoLayout     *layout,
+	  gint            *x_offset,
+	  gint            *y_offset,
+	  gint            *width,
+	  gint            *height)
 {
   GtkCellRendererText *celltext = (GtkCellRendererText *) cell;
   PangoRectangle rect;
-  PangoLayout *layout;
   GtkCellRendererTextPrivate *priv;
 
   priv = GTK_CELL_RENDERER_TEXT_GET_PRIVATE (cell);
@@ -1378,33 +1508,36 @@ gtk_cell_renderer_text_get_size (GtkCellRenderer *cell,
 	return;
     }
   
-  layout = get_layout (celltext, widget, FALSE, 0);
+  if (layout)
+    g_object_ref (layout);
+  else
+    layout = get_layout (celltext, widget, FALSE, 0);
 
   pango_layout_get_pixel_extents (layout, NULL, &rect);
-  
+
   if (height)
-    *height = GTK_CELL_RENDERER (celltext)->ypad * 2 + rect.height;
+    *height = cell->ypad * 2 + rect.height;
 
   /* The minimum size for ellipsized labels is ~ 3 chars */
   if (width)
     {
-      if (priv->ellipsize)
+      if (priv->ellipsize || priv->width_chars > 0)
 	{
 	  PangoContext *context;
 	  PangoFontMetrics *metrics;
 	  gint char_width;
 
 	  context = pango_layout_get_context (layout);
-	  metrics = pango_context_get_metrics (context, widget->style->font_desc, NULL);
+	  metrics = pango_context_get_metrics (context, widget->style->font_desc, pango_context_get_language (context));
 
 	  char_width = pango_font_metrics_get_approximate_char_width (metrics);
 	  pango_font_metrics_unref (metrics);
-
-	  *width += (PANGO_PIXELS (char_width) * 3);
+	  
+	  *width = cell->xpad * 2 + (PANGO_PIXELS (char_width) * MAX (priv->width_chars, 3));
 	}
       else
 	{
-	  *width = GTK_CELL_RENDERER (celltext)->xpad * 2 + rect.width;
+	  *width = cell->xpad * 2 + rect.x + rect.width;
 	}	  
     }
 
@@ -1413,16 +1546,16 @@ gtk_cell_renderer_text_get_size (GtkCellRenderer *cell,
       if (x_offset)
 	{
 	  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
-	    *x_offset = (1.0 - cell->xalign) * (cell_area->width - rect.width - (2 * cell->xpad));
+	    *x_offset = (1.0 - cell->xalign) * (cell_area->width - (rect.x + rect.width + (2 * cell->xpad)));
 	  else 
-	    *x_offset = cell->xalign * (cell_area->width - rect.width - (2 * cell->xpad));
+	    *x_offset = cell->xalign * (cell_area->width - (rect.x + rect.width + (2 * cell->xpad)));
 
-	  if (priv->ellipsize)
+	  if (priv->ellipsize_set || priv->wrap_width != -1)
 	    *x_offset = MAX(*x_offset, 0);
 	}
       if (y_offset)
 	{
-	  *y_offset = cell->yalign * (cell_area->height - rect.height - (2 * cell->ypad));
+	  *y_offset = cell->yalign * (cell_area->height - (rect.height + (2 * cell->ypad)));
 	  *y_offset = MAX (*y_offset, 0);
 	}
     }
@@ -1430,6 +1563,20 @@ gtk_cell_renderer_text_get_size (GtkCellRenderer *cell,
   g_object_unref (layout);
 }
 
+
+static void
+gtk_cell_renderer_text_get_size (GtkCellRenderer *cell,
+				 GtkWidget       *widget,
+				 GdkRectangle    *cell_area,
+				 gint            *x_offset,
+				 gint            *y_offset,
+				 gint            *width,
+				 gint            *height)
+{
+  get_size (cell, widget, cell_area, NULL,
+	    x_offset, y_offset, width, height);
+}
+
 static void
 gtk_cell_renderer_text_render (GtkCellRenderer      *cell,
 			       GdkDrawable          *window,
@@ -1450,8 +1597,7 @@ gtk_cell_renderer_text_render (GtkCellRenderer      *cell,
   priv = GTK_CELL_RENDERER_TEXT_GET_PRIVATE (cell);
 
   layout = get_layout (celltext, widget, TRUE, flags);
-
-  gtk_cell_renderer_text_get_size (cell, widget, cell_area, &x_offset, &y_offset, NULL, NULL);
+  get_size (cell, widget, cell_area, layout, &x_offset, &y_offset, NULL, NULL);
 
   if (!cell->sensitive) 
     {
@@ -1477,39 +1623,33 @@ gtk_cell_renderer_text_render (GtkCellRenderer      *cell,
 	state = GTK_STATE_NORMAL;
     }
 
-  if (celltext->background_set && state != GTK_STATE_SELECTED)
+  if (celltext->background_set && 
+      (flags & GTK_CELL_RENDERER_SELECTED) == 0)
     {
-      GdkColor color;
-      GdkGC *gc;
+      cairo_t *cr = gdk_cairo_create (window);
+
+      if (expose_area)
+	{
+	  gdk_cairo_rectangle (cr, expose_area);
+	  cairo_clip (cr);
+	}
+
+      gdk_cairo_rectangle (cr, background_area);
+      cairo_set_source_rgb (cr,
+			    celltext->background.red / 65535.,
+			    celltext->background.green / 65535.,
+			    celltext->background.blue / 65535.);
+      cairo_fill (cr);
       
-      color.red = celltext->background.red;
-      color.green = celltext->background.green;
-      color.blue = celltext->background.blue;
-
-      gc = gdk_gc_new (window);
-
-      gdk_gc_set_rgb_fg_color (gc, &color);
-
-      if (expose_area)               
-        gdk_gc_set_clip_rectangle (gc, expose_area);
-      gdk_draw_rectangle (window,
-                          gc,
-                          TRUE,
-                          background_area->x,
-                          background_area->y,
-                          background_area->width,
-                          background_area->height);
-      if (expose_area)               
-        gdk_gc_set_clip_rectangle (gc, NULL);
-      g_object_unref (gc);
+      cairo_destroy (cr);
     }
 
-  if (priv->ellipsize)
+  if (priv->ellipsize_set)
     pango_layout_set_width (layout, 
 			    (cell_area->width - x_offset - 2 * cell->xpad) * PANGO_SCALE);
-  else
+  else if (priv->wrap_width == -1)
     pango_layout_set_width (layout, -1);
-  
+
   gtk_paint_layout (widget->style,
                     window,
                     state,
@@ -1554,11 +1694,10 @@ gtk_cell_renderer_text_editing_done (GtkCellEditable *entry,
       priv->entry_menu_popdown_timeout = 0;
     }
 
+  gtk_cell_renderer_stop_editing (GTK_CELL_RENDERER (data), 
+				  GTK_ENTRY (entry)->editing_canceled);
   if (GTK_ENTRY (entry)->editing_canceled)
-    {
-      gtk_cell_renderer_editing_canceled (GTK_CELL_RENDERER (data));
-      return;
-    }
+    return;
 
   path = g_object_get_data (G_OBJECT (entry), GTK_CELL_RENDERER_TEXT_PATH);
   new_text = gtk_entry_get_text (GTK_ENTRY (entry));
@@ -1661,7 +1800,7 @@ gtk_cell_renderer_text_start_editing (GtkCellRenderer      *cell,
     return NULL;
 
   priv->entry = g_object_new (GTK_TYPE_ENTRY,
-			      "has_frame", FALSE,
+			      "has-frame", FALSE,
 			      "xalign", cell->xalign,
 			      NULL);
 
@@ -1728,3 +1867,6 @@ gtk_cell_renderer_text_set_fixed_height_from_font (GtkCellRendererText *renderer
       renderer->calc_fixed_height = TRUE;
     }
 }
+
+#define __GTK_CELL_RENDERER_TEXT_C__
+#include "gtkaliasdef.c"
