@@ -19,12 +19,12 @@
  */
 
 #include <config.h>
-#include "gtkalias.h"
 #include "gtkfilechooserwidget.h"
 #include "gtkfilechooserdefault.h"
 #include "gtkfilechooserutils.h"
 #include "gtktypebuiltins.h"
 #include "gtkfilechooserembed.h"
+#include "gtkalias.h"
 
 struct _GtkFileChooserWidgetPrivate
 {
@@ -161,7 +161,7 @@ gtk_file_chooser_widget_constructor (GType                  type,
   current_folder = g_get_current_dir ();
   gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (priv->impl), current_folder);
   g_free (current_folder);
-  
+
   _gtk_file_chooser_set_delegate (GTK_FILE_CHOOSER (object),
 				  GTK_FILE_CHOOSER (priv->impl));
 
@@ -248,3 +248,6 @@ gtk_file_chooser_widget_new_with_backend (GtkFileChooserAction  action,
 		       "file-system-backend", backend,
 		       NULL);
 }
+
+#define __GTK_FILE_CHOOSER_WIDGET_C__
+#include "gtkaliasdef.c"
