@@ -26,19 +26,24 @@
 
 #include <config.h>
 #include <math.h>
+#include <string.h>
+
 #include "gtkcontainer.h"
 #include "gtkimage.h"
 #include "gtkiconfactory.h"
 #include "gtkstock.h"
 #include "gtkicontheme.h"
 #include "gtkintl.h"
+#include "gtkprivate.h"
 #include "gtkalias.h"
-#include <string.h>
 
 typedef struct _GtkImagePrivate GtkImagePrivate;
 
 struct _GtkImagePrivate
 {
+  /* Only used with GTK_IMAGE_ANIMATION, GTK_IMAGE_PIXBUF */
+  gchar *filename;
+
   gint pixel_size;
 };
 
@@ -60,7 +65,6 @@ static void gtk_image_style_set    (GtkWidget      *widget,
 static void gtk_image_screen_changed (GtkWidget    *widget,
 				      GdkScreen    *prev_screen);
 static void gtk_image_destroy      (GtkObject      *object);
-static void gtk_image_clear        (GtkImage       *image);
 static void gtk_image_reset        (GtkImage       *image);
 static void gtk_image_calc_size    (GtkImage       *image);
 
@@ -158,7 +162,7 @@ gtk_image_class_init (GtkImageClass *class)
                                                         P_("Pixbuf"),
                                                         P_("A GdkPixbuf to display"),
                                                         GDK_TYPE_PIXBUF,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_PIXMAP,
@@ -166,7 +170,7 @@ gtk_image_class_init (GtkImageClass *class)
                                                         P_("Pixmap"),
                                                         P_("A GdkPixmap to display"),
                                                         GDK_TYPE_PIXMAP,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_IMAGE,
@@ -174,7 +178,7 @@ gtk_image_class_init (GtkImageClass *class)
                                                         P_("Image"),
                                                         P_("A GdkImage to display"),
                                                         GDK_TYPE_IMAGE,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_MASK,
@@ -182,7 +186,7 @@ gtk_image_class_init (GtkImageClass *class)
                                                         P_("Mask"),
                                                         P_("Mask bitmap to use with GdkImage or GdkPixmap"),
                                                         GDK_TYPE_PIXMAP,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_FILE,
@@ -190,7 +194,7 @@ gtk_image_class_init (GtkImageClass *class)
                                                         P_("Filename"),
                                                         P_("Filename to load and display"),
                                                         NULL,
-                                                        G_PARAM_WRITABLE));
+                                                        GTK_PARAM_READWRITE));
   
 
   g_object_class_install_property (gobject_class,
@@ -199,24 +203,24 @@ gtk_image_class_init (GtkImageClass *class)
                                                         P_("Stock ID"),
                                                         P_("Stock ID for a stock image to display"),
                                                         NULL,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_ICON_SET,
-                                   g_param_spec_boxed ("icon_set",
+                                   g_param_spec_boxed ("icon-set",
                                                        P_("Icon set"),
                                                        P_("Icon set to display"),
                                                        GTK_TYPE_ICON_SET,
-                                                       G_PARAM_READWRITE));
+                                                       GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_ICON_SIZE,
-                                   g_param_spec_int ("icon_size",
+                                   g_param_spec_int ("icon-size",
                                                      P_("Icon size"),
                                                      P_("Symbolic size to use for stock icon, icon set or named icon"),
                                                      0, G_MAXINT,
                                                      DEFAULT_ICON_SIZE,
-                                                     G_PARAM_READWRITE));
+                                                     GTK_PARAM_READWRITE));
   /**
    * GtkImage:pixel-size:
    *
@@ -228,20 +232,20 @@ gtk_image_class_init (GtkImageClass *class)
    */
   g_object_class_install_property (gobject_class,
 				   PROP_PIXEL_SIZE,
-				   g_param_spec_int ("pixel_size",
+				   g_param_spec_int ("pixel-size",
 						     P_("Pixel size"),
 						     P_("Pixel size to use for named icon"),
 						     -1, G_MAXINT,
 						     -1,
-						     G_PARAM_READWRITE));
+						     GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_PIXBUF_ANIMATION,
-                                   g_param_spec_object ("pixbuf_animation",
+                                   g_param_spec_object ("pixbuf-animation",
                                                         P_("Animation"),
                                                         P_("GdkPixbufAnimation to display"),
                                                         GDK_TYPE_PIXBUF_ANIMATION,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
   /**
    * GtkImage:icon-name:
@@ -253,20 +257,20 @@ gtk_image_class_init (GtkImageClass *class)
    */
   g_object_class_install_property (gobject_class,
                                    PROP_ICON_NAME,
-                                   g_param_spec_string ("icon_name",
+                                   g_param_spec_string ("icon-name",
                                                         P_("Icon Name"),
                                                         P_("The name of the icon from the icon theme"),
                                                         NULL,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
                                    PROP_STORAGE_TYPE,
-                                   g_param_spec_enum ("storage_type",
+                                   g_param_spec_enum ("storage-type",
                                                       P_("Storage type"),
                                                       P_("The representation being used for image data"),
                                                       GTK_TYPE_IMAGE_TYPE,
                                                       GTK_IMAGE_EMPTY,
-                                                      G_PARAM_READABLE));
+                                                      GTK_PARAM_READABLE));
 
   g_type_class_add_private (object_class, sizeof (GtkImagePrivate));
 }
@@ -283,6 +287,8 @@ gtk_image_init (GtkImage *image)
   image->mask = NULL;
 
   priv->pixel_size = -1;
+
+  priv->filename = NULL;
 }
 
 static void
@@ -347,8 +353,7 @@ gtk_image_set_property (GObject      *object,
         }
       break;
     case PROP_FILE:
-      gtk_image_set_from_file (image,
-                               g_value_get_string (value));
+      gtk_image_set_from_file (image, g_value_get_string (value));
       break;
     case PROP_STOCK:
       gtk_image_set_from_stock (image, g_value_get_string (value),
@@ -437,6 +442,9 @@ gtk_image_get_property (GObject     *object,
         g_value_set_object (value,
                             image->data.image.image);
       break;
+    case PROP_FILE:
+      g_value_set_string (value, priv->filename);
+      break;
     case PROP_STOCK:
       if (image->storage_type != GTK_IMAGE_STOCK)
         g_value_set_string (value, NULL);
@@ -605,7 +613,7 @@ gtk_image_new_from_pixbuf (GdkPixbuf *pixbuf)
  * Creates a #GtkImage displaying a stock icon. Sample stock icon
  * names are #GTK_STOCK_OPEN, #GTK_STOCK_EXIT. Sample stock sizes
  * are #GTK_ICON_SIZE_MENU, #GTK_ICON_SIZE_SMALL_TOOLBAR. If the stock
- * icon name isn't known, a "broken image" icon will be displayed instead.
+ * icon name isn't known, the image will be empty.
  * You can register your own stock icon names, see
  * gtk_icon_factory_add_default() and gtk_icon_factory_add().
  * 
@@ -825,6 +833,7 @@ void
 gtk_image_set_from_file   (GtkImage    *image,
                            const gchar *filename)
 {
+  GtkImagePrivate *priv = GTK_IMAGE_GET_PRIVATE (image);
   GdkPixbufAnimation *anim;
   
   g_return_if_fail (GTK_IS_IMAGE (image));
@@ -835,10 +844,11 @@ gtk_image_set_from_file   (GtkImage    *image,
 
   if (filename == NULL)
     {
+      priv->filename = NULL;
       g_object_thaw_notify (G_OBJECT (image));
       return;
     }
-  
+
   anim = gdk_pixbuf_animation_new_from_file (filename, NULL);
 
   if (anim == NULL)
@@ -856,17 +866,15 @@ gtk_image_set_from_file   (GtkImage    *image,
    */
 
   if (gdk_pixbuf_animation_is_static_image (anim))
-    {
-      gtk_image_set_from_pixbuf (image,
-                                 gdk_pixbuf_animation_get_static_image (anim));
-    }
+    gtk_image_set_from_pixbuf (image,
+			       gdk_pixbuf_animation_get_static_image (anim));
   else
-    {
-      gtk_image_set_from_animation (image, anim);
-    }
+    gtk_image_set_from_animation (image, anim);
 
   g_object_unref (anim);
 
+  priv->filename = g_strdup (filename);
+  
   g_object_thaw_notify (G_OBJECT (image));
 }
 
@@ -1545,10 +1553,7 @@ gtk_image_expose (GtkWidget      *widget,
       GdkBitmap *mask;
       GdkPixbuf *pixbuf;
       gboolean needs_state_transform;
-      GtkStockItem item;
-      gchar *stock_id;
-	  
-      
+
       image = GTK_IMAGE (widget);
       misc = GTK_MISC (widget);
 
@@ -1571,11 +1576,9 @@ gtk_image_expose (GtkWidget      *widget,
 	xalign = 1.0 - misc->xalign;
   
       x = floor (widget->allocation.x + misc->xpad
-		 + ((widget->allocation.width - widget->requisition.width) * xalign)
-		 + 0.5);
+		 + ((widget->allocation.width - widget->requisition.width) * xalign));
       y = floor (widget->allocation.y + misc->ypad 
-		 + ((widget->allocation.height - widget->requisition.height) * misc->yalign)
-		 + 0.5);
+		 + ((widget->allocation.height - widget->requisition.height) * misc->yalign));
       mask_x = x;
       mask_y = y;
       
@@ -1655,12 +1658,8 @@ gtk_image_expose (GtkWidget      *widget,
           break;
 
         case GTK_IMAGE_STOCK:
-	  if (gtk_stock_lookup (image->data.stock.stock_id, &item))
-	    stock_id = image->data.stock.stock_id;
-	  else
-	    stock_id = GTK_STOCK_MISSING_IMAGE;
           pixbuf = gtk_widget_render_icon (widget,
-                                           stock_id,
+                                           image->data.stock.stock_id,
                                            image->icon_size,
                                            NULL);
           if (pixbuf)
@@ -1837,9 +1836,23 @@ gtk_image_expose (GtkWidget      *widget,
   return FALSE;
 }
 
-static void
+/**
+ * gtk_image_clear:
+ * @image: a #GtkImage
+ *
+ * Resets the image to be empty.
+ *
+ * Since: 2.8
+ */
+void
 gtk_image_clear (GtkImage *image)
 {
+  GtkImagePrivate *priv;
+
+  g_return_if_fail (GTK_IS_IMAGE (image));
+
+  priv = GTK_IMAGE_GET_PRIVATE (image);
+
   g_object_freeze_notify (G_OBJECT (image));
   
   if (image->storage_type != GTK_IMAGE_EMPTY)
@@ -1935,6 +1948,13 @@ gtk_image_clear (GtkImage *image)
       
     }
 
+  if (priv->filename)
+    {
+      g_free (priv->filename);
+      priv->filename = NULL;
+      g_object_notify (G_OBJECT (image), "file");
+    }
+
   image->storage_type = GTK_IMAGE_EMPTY;
 
   memset (&image->data, '\0', sizeof (image->data));
@@ -1955,8 +1975,6 @@ gtk_image_calc_size (GtkImage *image)
 {
   GtkWidget *widget = GTK_WIDGET (image);
   GdkPixbuf *pixbuf = NULL;
-  GtkStockItem item;
-  gchar *stock_id;
   
   /* We update stock/icon set on every size request, because
    * the theme could have affected the size; for other kinds of
@@ -1966,12 +1984,8 @@ gtk_image_calc_size (GtkImage *image)
   switch (image->storage_type)
     {
     case GTK_IMAGE_STOCK:
-      if (gtk_stock_lookup (image->data.stock.stock_id, &item))
-	stock_id = image->data.stock.stock_id;
-      else
-	stock_id = GTK_STOCK_MISSING_IMAGE;
       pixbuf = gtk_widget_render_icon (widget,
-                                       stock_id,
+				       image->data.stock.stock_id,
                                        image->icon_size,
                                        NULL);
       break;
