@@ -35,7 +35,9 @@
 #endif
 
 #include "gtkprogressbar.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 
 #define MIN_HORIZONTAL_BAR_WIDTH   150
@@ -53,6 +55,7 @@ enum {
   PROP_PULSE_STEP,
   PROP_ORIENTATION,
   PROP_TEXT,
+  PROP_ELLIPSIZE,
   
   /* Deprecated args */
   PROP_ADJUSTMENT,
@@ -139,86 +142,109 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_ADJUSTMENT,
                                    g_param_spec_object ("adjustment",
-                                                        _("Adjustment"),
-                                                        _("The GtkAdjustment connected to the progress bar (Deprecated)"),
+                                                        P_("Adjustment"),
+                                                        P_("The GtkAdjustment connected to the progress bar (Deprecated)"),
                                                         GTK_TYPE_ADJUSTMENT,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ORIENTATION,
                                    g_param_spec_enum ("orientation",
-						      _("Orientation"),
-						      _("Orientation and growth direction of the progress bar"),
+						      P_("Orientation"),
+						      P_("Orientation and growth direction of the progress bar"),
 						      GTK_TYPE_PROGRESS_BAR_ORIENTATION,
 						      GTK_PROGRESS_LEFT_TO_RIGHT,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_BAR_STYLE,
-                                   g_param_spec_enum ("bar_style",
-						      _("Bar style"),
-						      _("Specifies the visual style of the bar in percentage mode (Deprecated)"),
+                                   g_param_spec_enum ("bar-style",
+						      P_("Bar style"),
+						      P_("Specifies the visual style of the bar in percentage mode (Deprecated)"),
 						      GTK_TYPE_PROGRESS_BAR_STYLE,
 						      GTK_PROGRESS_CONTINUOUS,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVITY_STEP,
-                                   g_param_spec_uint ("activity_step",
-						      _("Activity Step"),
-						      _("The increment used for each iteration in activity mode (Deprecated)"),
+                                   g_param_spec_uint ("activity-step",
+						      P_("Activity Step"),
+						      P_("The increment used for each iteration in activity mode (Deprecated)"),
 						      -G_MAXUINT,
 						      G_MAXUINT,
 						      3,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVITY_BLOCKS,
-                                   g_param_spec_uint ("activity_blocks",
-						      _("Activity Blocks"),
-						      _("The number of blocks which can fit in the progress bar area in activity mode (Deprecated)"),
+                                   g_param_spec_uint ("activity-blocks",
+						      P_("Activity Blocks"),
+						      P_("The number of blocks which can fit in the progress bar area in activity mode (Deprecated)"),
 						      2,
 						      G_MAXUINT,
 						      5,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_DISCRETE_BLOCKS,
-                                   g_param_spec_uint ("discrete_blocks",
-						      _("Discrete Blocks"),
-						      _("The number of discrete blocks in a progress bar (when shown in the discrete style)"),
+                                   g_param_spec_uint ("discrete-blocks",
+						      P_("Discrete Blocks"),
+						      P_("The number of discrete blocks in a progress bar (when shown in the discrete style)"),
 						      2,
 						      G_MAXUINT,
 						      10,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
 				   PROP_FRACTION,
 				   g_param_spec_double ("fraction",
-							_("Fraction"),
-							_("The fraction of total work that has been completed"),
+							P_("Fraction"),
+							P_("The fraction of total work that has been completed"),
 							0.0,
 							1.0,
 							0.0,
-							G_PARAM_READWRITE));  
+							GTK_PARAM_READWRITE));  
   
   g_object_class_install_property (gobject_class,
 				   PROP_PULSE_STEP,
-				   g_param_spec_double ("pulse_step",
-							_("Pulse Step"),
-							_("The fraction of total progress to move the bouncing block when pulsed"),
+				   g_param_spec_double ("pulse-step",
+							P_("Pulse Step"),
+							P_("The fraction of total progress to move the bouncing block when pulsed"),
 							0.0,
 							1.0,
 							0.1,
-							G_PARAM_READWRITE));  
+							GTK_PARAM_READWRITE));  
   
   g_object_class_install_property (gobject_class,
 				   PROP_TEXT,
 				   g_param_spec_string ("text",
-							_("Text"),
-							_("Text to be displayed in the progress bar"),
+							P_("Text"),
+							P_("Text to be displayed in the progress bar"),
 							"%P %%",
-							G_PARAM_READWRITE));
+							GTK_PARAM_READWRITE));
+
+  /**
+   * GtkProgressBar:ellipsize:
+   *
+   * The preferred place to ellipsize the string, if the progressbar does 
+   * not have enough room to display the entire string, specified as a 
+   * #PangoEllisizeMode. 
+   *
+   * Note that setting this property to a value other than 
+   * %PANGO_ELLIPSIZE_NONE has the side-effect that the progressbar requests 
+   * only enough space to display the ellipsis "...". Another means to set a 
+   * progressbar's width is gtk_widget_set_size_request().
+   *
+   * Since: 2.6
+   */
+  g_object_class_install_property (gobject_class,
+				   PROP_ELLIPSIZE,
+                                   g_param_spec_enum ("ellipsize",
+                                                      P_("Ellipsize"),
+                                                      P_("The preferred place to ellipsize the string, if the progressbar does not have enough room to display the entire string, if at all"),
+						      PANGO_TYPE_ELLIPSIZE_MODE,
+						      PANGO_ELLIPSIZE_NONE,
+                                                      GTK_PARAM_READWRITE));
 
 }
 
@@ -234,6 +260,7 @@ gtk_progress_bar_init (GtkProgressBar *pbar)
   pbar->activity_dir = 1;
   pbar->activity_step = 3;
   pbar->activity_blocks = 5;
+  pbar->ellipsize = PANGO_ELLIPSIZE_NONE;
 }
 
 static void
@@ -276,6 +303,9 @@ gtk_progress_bar_set_property (GObject      *object,
     case PROP_TEXT:
       gtk_progress_bar_set_text (pbar, g_value_get_string (value));
       break;
+    case PROP_ELLIPSIZE:
+      gtk_progress_bar_set_ellipsize (pbar, g_value_get_enum (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -321,6 +351,9 @@ gtk_progress_bar_get_property (GObject      *object,
     case PROP_TEXT:
       g_value_set_string (value, gtk_progress_bar_get_text (pbar));
       break;
+    case PROP_ELLIPSIZE:
+      g_value_set_enum (value, pbar->ellipsize);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -461,6 +494,7 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
   gchar *buf;
   PangoRectangle logical_rect;
   PangoLayout *layout;
+  gint width, height;
 
   g_return_if_fail (GTK_IS_PROGRESS_BAR (widget));
   g_return_if_fail (requisition != NULL);
@@ -468,6 +502,9 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
   progress = GTK_PROGRESS (widget);
   pbar = GTK_PROGRESS_BAR (widget);
 
+  width = 2 * widget->style->xthickness + 3 + 2 * TEXT_SPACING;
+  height = 2 * widget->style->ythickness + 3 + 2 * TEXT_SPACING;
+
   if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
     {
       if (!progress->adjustment)
@@ -476,8 +513,29 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
       buf = gtk_progress_get_text_from_value (progress, progress->adjustment->upper);
 
       layout = gtk_widget_create_pango_layout (widget, buf);
+
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 	  
+      if (pbar->ellipsize)
+	{
+	  PangoContext *context;
+	  PangoFontMetrics *metrics;
+	  gint char_width;
+	  
+	  /* The minimum size for ellipsized text is ~ 3 chars */
+	  context = pango_layout_get_context (layout);
+	  metrics = pango_context_get_metrics (context, widget->style->font_desc, pango_context_get_language (context));
+	  
+	  char_width = pango_font_metrics_get_approximate_char_width (metrics);
+	  pango_font_metrics_unref (metrics);
+	  
+	  width += PANGO_PIXELS (char_width) * 3;
+	}
+      else
+	width += logical_rect.width;
+      
+      height += logical_rect.height;
+
       g_object_unref (layout);
       g_free (buf);
     }
@@ -487,13 +545,8 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
     {
       if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
 	{
-	  requisition->width = MAX (MIN_HORIZONTAL_BAR_WIDTH,
-				    2 * widget->style->xthickness + 3 +
-				    logical_rect.width + 2 * TEXT_SPACING);
-
-	  requisition->height = MAX (MIN_HORIZONTAL_BAR_HEIGHT,
-				     2 * widget->style->ythickness + 3 +
-				     logical_rect.height + 2 * TEXT_SPACING);
+	  requisition->width = MAX (MIN_HORIZONTAL_BAR_WIDTH, width);
+	  requisition->height = MAX (MIN_HORIZONTAL_BAR_HEIGHT, height);
 	}
       else
 	{
@@ -505,13 +558,8 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
     {
       if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
 	{	  
-	  requisition->width = MAX (MIN_VERTICAL_BAR_WIDTH,
-				    2 * widget->style->xthickness + 3 +
-				    logical_rect.width + 2 * TEXT_SPACING);
-
-	  requisition->height = MAX (MIN_VERTICAL_BAR_HEIGHT,
-				     2 * widget->style->ythickness + 3 +
-				     logical_rect.height + 2 * TEXT_SPACING);
+	  requisition->width = MAX (MIN_VERTICAL_BAR_WIDTH, width);
+	  requisition->height = MAX (MIN_VERTICAL_BAR_HEIGHT, height);
 	}
       else
 	{
@@ -736,6 +784,10 @@ gtk_progress_bar_paint_text (GtkProgressBar            *pbar,
   buf = gtk_progress_get_current_text (progress);
   
   layout = gtk_widget_create_pango_layout (widget, buf);
+  pango_layout_set_ellipsize (layout, pbar->ellipsize);
+  if (pbar->ellipsize)
+    pango_layout_set_width (layout, widget->allocation.width * PANGO_SCALE);
+
   pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
   
   x = widget->style->xthickness + 1 +
@@ -882,7 +934,7 @@ gtk_progress_bar_set_bar_style_internal (GtkProgressBar     *pbar,
       if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
 	gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
-      g_object_notify (G_OBJECT (pbar), "bar_style");
+      g_object_notify (G_OBJECT (pbar), "bar-style");
     }
 }
 
@@ -900,7 +952,7 @@ gtk_progress_bar_set_discrete_blocks_internal (GtkProgressBar *pbar,
       if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
 	gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
-      g_object_notify (G_OBJECT (pbar), "discrete_blocks");
+      g_object_notify (G_OBJECT (pbar), "discrete-blocks");
     }
 }
 
@@ -913,7 +965,7 @@ gtk_progress_bar_set_activity_step_internal (GtkProgressBar *pbar,
   if (pbar->activity_step != step)
     {
       pbar->activity_step = step;
-      g_object_notify (G_OBJECT (pbar), "activity_step");
+      g_object_notify (G_OBJECT (pbar), "activity-step");
     }
 }
 
@@ -927,7 +979,7 @@ gtk_progress_bar_set_activity_blocks_internal (GtkProgressBar *pbar,
   if (pbar->activity_blocks != blocks)
     {
       pbar->activity_blocks = blocks;
-      g_object_notify (G_OBJECT (pbar), "activity_blocks");
+      g_object_notify (G_OBJECT (pbar), "activity-blocks");
     }
 }
 
@@ -1024,7 +1076,7 @@ gtk_progress_bar_set_pulse_step   (GtkProgressBar *pbar,
   
   pbar->pulse_fraction = fraction;
 
-  g_object_notify (G_OBJECT (pbar), "pulse_step");
+  g_object_notify (G_OBJECT (pbar), "pulse-step");
 }
 
 void
@@ -1173,3 +1225,52 @@ gtk_progress_bar_set_activity_blocks (GtkProgressBar *pbar,
 
   gtk_progress_bar_set_activity_blocks_internal (pbar, blocks);
 }
+
+/**
+ * gtk_progress_bar_set_ellipsize:
+ * @pbar: a #GtkProgressBar
+ * @mode: a #PangoEllipsizeMode
+ *
+ * Sets the mode used to ellipsize (add an ellipsis: "...") the text 
+ * if there is not enough space to render the entire string.
+ *
+ * Since: 2.6
+ **/
+void
+gtk_progress_bar_set_ellipsize (GtkProgressBar     *pbar,
+				PangoEllipsizeMode  mode)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
+  g_return_if_fail (mode >= PANGO_ELLIPSIZE_NONE && 
+		    mode <= PANGO_ELLIPSIZE_END);
+  
+  if ((PangoEllipsizeMode)pbar->ellipsize != mode)
+    {
+      pbar->ellipsize = mode;
+
+      g_object_notify (G_OBJECT (pbar), "ellipsize");
+      gtk_widget_queue_resize (GTK_WIDGET (pbar));
+    }
+}
+
+/**
+ * gtk_progress_bar_get_ellipsize:
+ * @pbar: a #GtkProgressBar
+ *
+ * Returns the ellipsizing position of the progressbar. 
+ * See gtk_progress_bar_set_ellipsize().
+ *
+ * Return value: #PangoEllipsizeMode
+ *
+ * Since: 2.6
+ **/
+PangoEllipsizeMode 
+gtk_progress_bar_get_ellipsize (GtkProgressBar *pbar)
+{
+  g_return_val_if_fail (GTK_IS_PROGRESS_BAR (pbar), PANGO_ELLIPSIZE_NONE);
+
+  return pbar->ellipsize;
+}
+
+#define __GTK_PROGRESS_BAR_C__
+#include "gtkaliasdef.c"
