@@ -33,9 +33,7 @@
 #include <gtk/gtkwidget.h>
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 
 #define GTK_TYPE_RANGE            (gtk_range_get_type ())
@@ -120,11 +118,14 @@ struct _GtkRangeClass
   void (* get_range_border) (GtkRange     *range,
                              GtkBorder    *border_);
 
+  gboolean (* change_value) (GtkRange     *range,
+                             GtkScrollType scroll,
+                             gdouble       new_value);
+
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
   void (*_gtk_reserved3) (void);
-  void (*_gtk_reserved4) (void);
 };
 
 
@@ -149,10 +150,11 @@ void           gtk_range_set_value         (GtkRange      *range,
                                             gdouble        value);
 gdouble        gtk_range_get_value         (GtkRange      *range);
 
+gdouble	      _gtk_range_get_wheel_delta   (GtkRange           *range,
+					    GdkScrollDirection  direction);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+
+G_END_DECLS
 
 
 #endif /* __GTK_RANGE_H__ */
