@@ -20,14 +20,14 @@
 #include <config.h>
 #include <string.h>
 #include <gobject/gvaluecollector.h>
-#include "gtkalias.h"
 #include "gtktreemodel.h"
 #include "gtktreestore.h"
 #include "gtktreedatalist.h"
 #include "gtktreednd.h"
+#include "gtkalias.h"
 
 #define G_NODE(node) ((GNode *)node)
-#define GTK_TREE_STORE_IS_SORTED(tree) (GTK_TREE_STORE (tree)->sort_column_id != -2)
+#define GTK_TREE_STORE_IS_SORTED(tree) (GTK_TREE_STORE (tree)->sort_column_id != GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID)
 #define VALID_ITER(iter, tree_store) (iter!= NULL && iter->user_data != NULL && tree_store->stamp == iter->stamp)
 
 static void         gtk_tree_store_init            (GtkTreeStore      *tree_store);
@@ -74,6 +74,8 @@ static void gtk_tree_store_set_column_type (GtkTreeStore *tree_store,
 					    gint          column,
 					    GType         type);
 
+static void gtk_tree_store_increment_stamp (GtkTreeStore  *tree_store);
+
 
 /* DND interfaces */
 static gboolean real_gtk_tree_store_row_draggable   (GtkTreeDragSource *drag_source,
@@ -1140,8 +1142,10 @@ gtk_tree_store_insert (GtkTreeStore *tree_store,
 {
   GtkTreePath *path;
   GNode *parent_node;
+  GNode *new_node;
 
   g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
+  g_return_if_fail (iter != NULL);
   if (parent)
     g_return_if_fail (VALID_ITER (parent, tree_store));
 
@@ -1152,13 +1156,24 @@ gtk_tree_store_insert (GtkTreeStore *tree_store,
 
   tree_store->columns_dirty = TRUE;
 
+  new_node = g_node_new (NULL);
+
   iter->stamp = tree_store->stamp;
-  iter->user_data = g_node_new (NULL);
-  g_node_insert (parent_node, position, G_NODE (iter->user_data));
+  iter->user_data = new_node;
+  g_node_insert (parent_node, position, new_node);
 
   path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), iter);
   gtk_tree_model_row_inserted (GTK_TREE_MODEL (tree_store), path, iter);
 
+  if (parent_node != tree_store->root)
+    {
+      if (new_node->prev == NULL && new_node->next == NULL)
+        {
+          gtk_tree_path_up (path);
+          gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (tree_store), path, parent);
+        }
+    }
+
   gtk_tree_path_free (path);
 
   validate_tree ((GtkTreeStore*)tree_store);
@@ -1199,10 +1214,6 @@ gtk_tree_store_insert_before (GtkTreeStore *tree_store,
   if (sibling != NULL)
     g_return_if_fail (VALID_ITER (sibling, tree_store));
 
-  tree_store->columns_dirty = TRUE;
-
-  new_node = g_node_new (NULL);
-
   if (parent == NULL && sibling == NULL)
     parent_node = tree_store->root;
   else if (parent == NULL)
@@ -1215,6 +1226,10 @@ gtk_tree_store_insert_before (GtkTreeStore *tree_store,
       parent_node = G_NODE (parent->user_data);
     }
 
+  tree_store->columns_dirty = TRUE;
+
+  new_node = g_node_new (NULL);
+
   g_node_insert_before (parent_node,
 			sibling ? G_NODE (sibling->user_data) : NULL,
                         new_node);
@@ -1225,6 +1240,20 @@ gtk_tree_store_insert_before (GtkTreeStore *tree_store,
   path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), iter);
   gtk_tree_model_row_inserted (GTK_TREE_MODEL (tree_store), path, iter);
 
+  if (parent_node != tree_store->root)
+    {
+      if (new_node->prev == NULL && new_node->next == NULL)
+        {
+          GtkTreeIter parent_iter;
+
+          parent_iter.stamp = tree_store->stamp;
+          parent_iter.user_data = parent_node;
+
+          gtk_tree_path_up (path);
+          gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (tree_store), path, &parent_iter);
+        }
+    }
+
   gtk_tree_path_free (path);
 
   validate_tree ((GtkTreeStore*)tree_store);
@@ -1265,10 +1294,6 @@ gtk_tree_store_insert_after (GtkTreeStore *tree_store,
   if (sibling != NULL)
     g_return_if_fail (VALID_ITER (sibling, tree_store));
 
-  tree_store->columns_dirty = TRUE;
-
-  new_node = g_node_new (NULL);
-
   if (parent == NULL && sibling == NULL)
     parent_node = tree_store->root;
   else if (parent == NULL)
@@ -1282,6 +1307,9 @@ gtk_tree_store_insert_after (GtkTreeStore *tree_store,
       parent_node = G_NODE (parent->user_data);
     }
 
+  tree_store->columns_dirty = TRUE;
+
+  new_node = g_node_new (NULL);
 
   g_node_insert_after (parent_node,
 		       sibling ? G_NODE (sibling->user_data) : NULL,
@@ -1293,6 +1321,20 @@ gtk_tree_store_insert_after (GtkTreeStore *tree_store,
   path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), iter);
   gtk_tree_model_row_inserted (GTK_TREE_MODEL (tree_store), path, iter);
 
+  if (parent_node != tree_store->root)
+    {
+      if (new_node->prev == NULL && new_node->next == NULL)
+        {
+          GtkTreeIter parent_iter;
+
+          parent_iter.stamp = tree_store->stamp;
+          parent_iter.user_data = parent_node;
+
+          gtk_tree_path_up (path);
+          gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (tree_store), path, &parent_iter);
+        }
+    }
+
   gtk_tree_path_free (path);
 
   validate_tree ((GtkTreeStore*)tree_store);
@@ -1501,6 +1543,16 @@ gtk_tree_store_clear_traverse (GNode *node,
   return FALSE;
 }
 
+static void
+gtk_tree_store_increment_stamp (GtkTreeStore *tree_store)
+{
+  do
+    {
+      tree_store->stamp++;
+    }
+  while (tree_store->stamp == 0);
+}
+
 /**
  * gtk_tree_store_clear:
  * @tree_store: a #GtkTreeStore
@@ -1513,6 +1565,7 @@ gtk_tree_store_clear (GtkTreeStore *tree_store)
   g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
 
   gtk_tree_store_clear_traverse (tree_store->root, tree_store);
+  gtk_tree_store_increment_stamp (tree_store);
 }
 
 static gboolean
@@ -2555,8 +2608,6 @@ gtk_tree_store_sort_helper (GtkTreeStore *tree_store,
       return;
     }
 
-  g_assert (GTK_TREE_STORE_IS_SORTED (tree_store));
-
   list_length = 0;
   for (tmp_node = node; tmp_node; tmp_node = tmp_node->next)
     list_length++;
@@ -2615,6 +2666,9 @@ gtk_tree_store_sort_helper (GtkTreeStore *tree_store,
 static void
 gtk_tree_store_sort (GtkTreeStore *tree_store)
 {
+  if (!GTK_TREE_STORE_IS_SORTED (tree_store))
+    return;
+
   if (tree_store->sort_column_id != -1)
     {
       GtkTreeDataSortHeader *header = NULL;
@@ -2832,15 +2886,16 @@ gtk_tree_store_get_sort_column_id (GtkTreeSortable  *sortable,
 
   g_return_val_if_fail (GTK_IS_TREE_STORE (sortable), FALSE);
 
-  if (tree_store->sort_column_id == GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
-    return FALSE;
-
   if (sort_column_id)
     * sort_column_id = tree_store->sort_column_id;
   if (order)
     * order = tree_store->order;
-  return TRUE;
 
+  if (tree_store->sort_column_id == GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID ||
+      tree_store->sort_column_id == GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID)
+    return FALSE;
+
+  return TRUE;
 }
 
 static void
@@ -2857,19 +2912,22 @@ gtk_tree_store_set_sort_column_id (GtkTreeSortable  *sortable,
       (tree_store->order == order))
     return;
 
-  if (sort_column_id != GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
+  if (sort_column_id != GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID)
     {
-      GtkTreeDataSortHeader *header = NULL;
+      if (sort_column_id != GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID)
+	{
+	  GtkTreeDataSortHeader *header = NULL;
 
-      header = _gtk_tree_data_list_get_header (tree_store->sort_list, sort_column_id);
+	  header = _gtk_tree_data_list_get_header (tree_store->sort_list, sort_column_id);
 
-      /* We want to make sure that we have a function */
-      g_return_if_fail (header != NULL);
-      g_return_if_fail (header->func != NULL);
-    }
-  else
-    {
-      g_return_if_fail (tree_store->default_sort_func != NULL);
+	  /* We want to make sure that we have a function */
+	  g_return_if_fail (header != NULL);
+	  g_return_if_fail (header->func != NULL);
+	}
+      else
+	{
+	  g_return_if_fail (tree_store->default_sort_func != NULL);
+	}
     }
 
   tree_store->sort_column_id = sort_column_id;
@@ -2980,3 +3038,6 @@ validate_gnode (GNode* node)
       iter = iter->next;
     }
 }
+
+#define __GTK_TREE_STORE_C__
+#include "gtkaliasdef.c"
