@@ -17,14 +17,15 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: imime.c,v 1.2 2004/10/01 19:53:55 tml Exp $
+ * $Id: imime.c,v 1.4 2005/07/06 00:31:45 matthiasc Exp $
  */
 
-#include <gtk/gtkimmodule.h>
-#include "gtkimcontextime.h"
-
 #include <string.h>
 
+#include "gtk/gtkintl.h"
+#include "gtk/gtkimmodule.h"
+#include "gtkimcontextime.h"
+
 static const GtkIMContextInfo ime_info = {
   "ime",
   "Windows IME",
@@ -60,7 +61,7 @@ im_module_create (const gchar * context_id)
 {
   g_return_val_if_fail (context_id, NULL);
 
-  if (!strcmp (context_id, "win32ime"))
+  if (!strcmp (context_id, "ime"))
     return g_object_new (GTK_TYPE_IM_CONTEXT_IME, NULL);
   else
     return NULL;
